# OBJ - timestamped JSON objects
#
# obj/cli/__init__.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 08-02-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" command line interface. """

## imports

from obj.base import cfg, Config, Object
from obj.fleet import Fleet
from obj.handler import Handler
from obj.loader import Loader
from obj.select import Select
from obj.shell import set_completer
from obj.users import Users
from obj.utils import services

import logging
import os
import sys

## vars

fleet = Fleet()
loader = Loader()
users = Users()

## classes

class CLI(Select):

    _sock = sys.stdin

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cfg = Config().fromdisk("cli")

    def announce(self, txt):
        if not self.cfg.silent:
            self.raw(txt)

    def cmd(self, txt):
        """ execute a command based on txt.  """
        name = txt.split()[0]
        logging.warn("cmd %s" % name)
        event = self.event(txt)
        event._batch = True
        event.origin = "root@shell"
        event.handle()
        return event

    def errored(self, event):
        logging.error(event)

    def event(self, txt=""):
        """ must return an Event and block for input function. """ 
        e = Object()
        e._loader = self._loader
        e._id = self._id
        e.origin = "root@shell"
        e.txt = txt or input()
        return e

    def handle(self, event):
        event.handle()
        self.prompt()

    def prompt(self, *args, **kwargs):
        """ echo prompt to sys.stdout. """
        sys.stdout.write("> ")
        sys.stdout.flush()

    def raw(self, txt):
        """ output to sys.stdout. """
        sys.stdout.write(txt)
        sys.stdout.write("\n")
        sys.stdout.flush()

    def reconnect(self, *args, **kwargs):
        """ connect to the server. """
        self._connected.ready()

    def start(self, loader=None):
        self._loader = loader or self
        if cfg.shell and cfg.banner:
            print("OBJ %s\n" % __version__)
        if cfg.write:
            cfg.save("config/main")
        elif cfg.fromfile:
            cfgfile = Config().fromdisk("main")
            cfg.update(cfgfile)
            logging.warn("restore %s" % cfgfile)
        users.oper("root@shell")
        if not os.path.isdir(cfg.workdir):
            cdir(cfg.workdir)
        if cfg.args:
            cli = CLI()
            e = cli.cmd(" ".join(cfg.args))
            e.wait()
            return
        if cfg.init:
            services()
        if cfg.shell:
            set_completer(self._loader._handlers.keys())
            fleet.register(self)
            self.register(self)
            super().start()
            self.prompt()
            self.wait()
