# OBJ - timestamped JSON objects
#
# obj/cli/cmds.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 16-05-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" basic bot commands. """

## imports

from obj.base import cfg as _cfg
from obj.base import Config, Object, __version__
from obj.decorator import oper
from obj.fleet import Fleet
from obj.shell import reset
from obj.shell import reboot as _reboot

import os
import json

## vars

fleet = Fleet()

## commands

def ed(event):
    """ object editor. """
    args = event._parsed.args
    if not args:
        event.reply("ed <name> <key> <value>")
        return
    fn = args[0]
    path = os.path.join(_cfg.workdir, fn)
    obj = Object().cache(path)
    if not obj:
        event.reply("no object %s found." % path)
        return
    if len(args) == 1:
        print(obj)
        return 
    key = args[1]
    if key not in obj:
        event.reply("EKEY %s" % key)
        return
    if len(args) == 2:
        event.reply('%s="%s"' % (key, obj[key]))
        return
    val = ""
    if len(args) > 3:
        val = args[2:]
    if len(args) == 3:
        val = args[2]
    try:
        val = int(val)
    except:
        try:
            val = float(val)
        except:
            pass
    if type(val) == list:
        obj[key] = list(val)
    elif val in ["True", "true"]:
        obj[key] = True
    elif val in ["False", "false"]:
        obj[key] = False
    else:
        try:
            val = '"%s"' % val
            obj[key] = json.loads(val)
        except (SyntaxError, ValueError) as ex:
            event.reply("%s: %s" % (str(ex), val))
            return
    event.reply('%s="%s"' % (key, obj[key]))
    p = obj.save()
    event.reply("ok %s" % p)

def cfg(event):
    """ show config file dump. """
    if event._parsed.args:
        c = Config().fromdisk(event._parsed.args[0])
    else:
        c = _cfg
    event.reply(c)

def ls(event):
    """ show subdirs in working directory. """
    if not os.path.isdir(_cfg.workdir):
        cdir(_cfg.workdir)
    prefixes = os.listdir(_cfg.workdir)
    if prefixes:
        event.reply(" ".join(prefixes))

def real_reboot(x):
    """ actual reboot. """
    reset()
    _reboot()

@oper
def reboot(event):
    """ perform a statefull reboot. """
    if not _cfg.reboot:
        event.reply("reboot is not enabled")
        return
    fleet.announce("reboot %s" % time.ctime(time.time()))
    fleet.save("run/fleet")
    real_reboot(event)

def show(event):
    import obj
    name = event._parsed.args
    if not name:
        event.reply("show <name>")
        return
    event.reply(find(name, obj))

def test(event):
    """ say hello. """
    event.reply("hello %s" % event.origin)

def version(event):
    """ show OBJ version. """
    event.reply("OBJ %s" % __version__)
