# OBJ - Framework to program bots
#
# obj/clock/__init__.py
#
# Copyright 2017,2018 B.H.J Thate
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice don't have to be included.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 08-02-2018 As the creator of this file, I disclaim all rights on  this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" timer, repeater. timed daemon. """

## imports

from obj.base import Config, Object, name
from obj.db import Db
from obj.thr import Launcher

import os
import logging
import threading
import time

## vars

_begintime = time.time()

db = Db()
launcher = Launcher()

## classes

class Timed(Object):
    """ Future Timer Object. """
    pass

class Timers(Object):

    """ run timers on their scheduled time. """

    def __init__(self, *args, **kwargs):
        self._name = "Timers"
        self._stopped = False
        self.cfg = Config()
        self.timers = Object()
        self.ready()

    def server(self):
        while not self._stopped:
            time.sleep(1.0)
            remove = []
            for t, event in self.timers.items():
                if time.time() > t:
                    self.cfg.latest = time.time()
                    self.cfg.save()
                    event.raw(event.txt)
                    remove.append(t)
            for r in remove:
                del self.timers[r]
        
    def start(self):
        """ search database for timers still running. """
        self.cfg.load("config/timers")
        for e in db.sequence("timed", 0, time.time() + 52*7*24*60*60):
            if "done" in e and e.done:
                continue
            if "time" not in e:
                continue
            if time.time() < int(e.time):
                self.timers[e.time] = e
        return launcher.launch(self.server)
        
    def stop(self):
        """ stop the timers logger. """
        self._stopped = True

class Timer(Object):

    """ call a function as x seconds of sleep. """

    def __init__(self, sleep, func, *args, **kwargs):
        super().__init__()
        self._func = func
        self._name = kwargs.get("name", name(self._func))
        self.sleep = sleep
        self.args = args
        self.kwargs = kwargs
        try:
            self._event = self.args[0]
        except:
            self._event = Object()
        self._time = Object()

    def start(self):
        """ start the timer. """
        timer = threading.Timer(self.sleep, self.run, self.args, self.kwargs)
        timer.setName(self._name)
        timer.sleep = self.sleep
        timer._event = self._event
        timer._time = self._time
        timer._time.start = time.time()
        timer._time.latest = time.time()
        timer._func = self._func
        timer.start()
        return timer

    def run(self, *args, **kwargs):
        """ run the registered function. """
        self._time.latest = time.time()
        self._func(*args, **kwarg)

    def exit(self):
        """ cancel the timer. """
        self.cancel()

class Repeater(Timer):

    """ repeat an funcion every x seconds. """

    def run(self, *args, **kwargs):
        self._func(*args, **kwargs)
        self.start()
