# OBJ - timestamped JSON objects
#
# obj/__init__.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 08-02-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    object class to save/load json files.
    if a pathname is not provided a timestamp filename is provided, for example:
    the cfg.workdir variable in this module is prepended to a filename.

"""

from obj.base import Object

class Collection(list):

    """ List of Objects. """


    def __iadd__(self, l):
        self.extend(list(l))
        return self

    def display(self, event):
        """ display the collection as a reply to an event. """
        nr = 0
        for obj in self:
            if "d" in event._parsed.enabled:
                event.reply(obj)
            else:
                event.display(obj, txt=str(nr))
            nr += 1

    def by_key(self, key, value=""):
        c = Collection()
        for obj in self:
            if key in self:
                o = getattr(obj, key, None)
                if o:
                   if value and not o == value:
                       continue
                       c.append(obj)
        return c

    def uniq(self, k):
        c = Collection()
        u = Object()
        for obj in self:
             v = getattr(obj, k)
             if k in u:
                 continue
             u[k] = v
             c.append(obj)
        return c
