# OBJBOT - Framework to program bots !!
#
# obj/decorator.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 9-1-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""  command decorator.  """

## imports

from obj.db import Db
from obj.users import Users

import functools

## vars

db = Db()
users = Users()

## decorators

def echofind(func, *args, **kwargs):

    """ if no arguments are provided show known values. """
    
    @functools.wraps(func)
    def check(*args, **kwargs):
        if args:
            event = args[0]
        if not event._parsed.rest:
            nr = 0
            for o in db.find(event._parsed.cmd):
                v = getattr(o, event._parsed.cmd, None)
                if v:
                    event.reply("%s %s" % (nr, v))
                    nr += 1
        return func(event)

    check.__name__ = func.__name__
    check.__doc__ = func.__doc__
    check.__wrapped__ = func
    return check

def echoattr(func, *args, **kwargs):
    
    """ if no arguments are provided show possible attributes. """
    
    @functools.wraps(func)
    def check(*args, **kwargs):
        if args:
            event = args[0]
        if not event._parsed.rest:
            event.reply(",".join([x for x in db.attr(event._parsed.cmd)]))
        return func(event)

    check.__name__ = func.__name__
    check.__doc__ = func.__doc__
    check.__wrapped__ = func
    return check

def oper(func, *args, **kwargs):
    
    """ see if user has OPER privileges. """ 
    
    @functools.wraps(func)
    def check(*args, **kwargs):
        event = args[0]
        if not users.allowed(event.origin, "OPER"):
            return
        return func(event)

    check.__name__ = func.__name__
    check.__doc__ = func.__doc__
    check.__wrapped__ = func
    return check

def uber(func, *args, **kwargs):

    """ check if user has UBER privileges. """
    
    @functools.wraps(func)
    def check(*args, **kwargs):
        event = args[0]
        if not users.allowed(event.origin, "UBER"):
            return
        return func(event)

    check.__name__ = func.__name__
    check.__doc__ = func.__doc__
    check.__wrapped__ = func
    return check
