# OBJ - timestamped JSON objects
#
# obj/loaded/cmds.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 06-04-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" commands module. """

## imports

from obj.decorator import oper
from obj.loader import Loader
from obj.thr import Launcher

## vars

launcher = Launcher()
loader = Loader()

@oper
def all(event):
    """ load all modules. """
    res = loader.walk("obj", True)
    event.reply(res)

def cmds(event):
    """ show commands registered. """
    event.reply(",".join(sorted(loader._handlers.keys())))

@oper
def kernel(event):
    """ display generic loader in obj.core. """
    event.reply(loader)

@oper
def reload(event):
    """ reload a plugin. """
    if not event._parsed.rest:
        event.reply(",".join([x.split(".")[-1] for x in loader.modules("obj")]))
        return
    for modname in loader.modules("obj"):
        if event._parsed.rest not in modname:
            continue
        try:
            mod = loader.reload(modname, False)
            if mod:
                event.reply("ok %s" % modname)
            else:
                event.reply("no %s module foudn to reload." % modname)
        except (AttributeError, KeyError) as ex:
            event.reply("%s %s" % (modname, str(ex)))

@oper
def start(event):
    """ start a plugin. """
    if not event._parsed.args:
        res = []
        for modname in loader.modules("obj"):
            mod = loader.direct(modname)
            if "start" in dir(mod):
                res.append(modname)
        event.reply("choose one of %s" % ",".join(sorted(res)))
        return
    n = event._parsed.args[0]
    mod = loader.direct(n)
    try:
        mod.start()
        event.reply("ok %s" % n)
    except:
        event.reply(get_exception())

@oper
def stop(event):
    """ stop a plugin. """
    if not event._parsed.rest:
        event.reply("stop what ?")
        return
    n  = event._parsed.args[0]
    if n:
        launcher.kill(n.upper())
