# OBJ - timestamped JSON objects
#
# obj/users/cmds.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 06-04-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" commands module. """

## imports

from obj.base import allowchar
from obj.decorator import oper
from obj.users import Users

## vars

users = Users()

@oper
def delperm(event):
    """ delete permissions of an user. """
    try:
        origin, perm = event._parsed.args
    except:
        event.reply("delperm <origin> <perm>")
        return
    origin = users._userhosts.get(origin, origin)
    result = users.delete(origin, perm)
    if not result:
        event.reply("can't find user %s" % origin)
        return
    event.reply("ok %s %s" % (origin, perm))
    
@oper
def meet(event):
    """ create an user record. """
    try:
        origin = event._parsed.rest
    except:
        event.reply("meet <nick> [<perm1> <perm2>]")
        return
    orig = origin.strip()
    if not allowchar(orig):
        event.reply("%s is not allowed as a username" % orig)
        return
    if orig:
        orig = users._userhosts.get(orig, orig)
        u = users.meet(orig)
        if u:
            event.reply("user %s created" % orig)
        else:
            event.reply("%s is already introduced." % orig)

def perm(event):
    """ add/change permissions of an user. """
    try:
        origin, perms = event._parsed.args
    except:
        event.reply("perm <origin> <perm>")
        return
    origin = users._userhosts.get(origin, origin)
    u = users.perm(origin, perms)
    if not u:
        event.reply("can't find a user matching %s" % origin)
        return
    event.reply("ok %s" % origin)

def perms(event):
    """ show permission of user. """
    origin = users._userhosts.get(event.origin, event.origin)
    u = users.fetch(origin)
    if u:
        event.reply("permissions are %s" % ",".join(u.perms))

def u(event):
    """ show user selected by userhost. """
    if not event._parsed.rest:
        event.reply("u <origin>")
        return
    origin = event._parsed.args[0]
    origin = users._userhosts.get(origin, origin)
    u = users.fetch(origin)
    if u:
        event.reply(u.pure())
    else:
        event.reply("no user %s found." % origin)

def w(event):
    """ show user data. """
    u = users.fetch(event.origin)
    event.reply(u or "no matching user %s found." % event.origin)

